@@ -6,6 +6,7 @@ NULL
 #' @rdname cutpoints-method
 #' @docType methods
 #' @exportMethod cutpoints
+#' @param object generic glmdisc object
 #' @author Adrien Ehrhardt.
 #' @description This defines the generic method "cutpoints" which will provide the cutpoints of a discretization scheme of S4 class \code{\link{glmdisc}}.
-methods::setGeneric("cutpoints", function(object,...) attributes(object))
\ No newline at end of file
+methods::setGeneric("cutpoints", function(object) attributes(object))
\ No newline at end of file
